# Generated by Django 4.0.3 on 2022-03-19 12:05

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('marketmanager', '0025_statisticsconfig'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='publicconfig',
            options={'default_permissions': (), 'verbose_name': 'Public Market Configuration'},
        ),
        migrations.AlterModelOptions(
            name='statisticsconfig',
            options={'default_permissions': (), 'verbose_name': 'TypeStatistics Calculation Configuration'},
        ),
    ]



# Generated by Django 4.0.3 on 2022-03-15 09:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('marketmanager', '0010_alter_watchconfig_debug_webhooks'),
    ]

    operations = [
        migrations.RenameField(
            model_name='watchconfig',
            old_name='location',
            new_name='structure',
        ),
    ]



# Generated by Django 4.0.3 on 2022-03-14 06:46

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('marketmanager', '0002_alter_structure_created_at_alter_watchconfig_type'),
    ]

    operations = [
        migrations.RenameField(
            model_name='watchconfig',
            old_name='type',
            new_name='eve_type',
        ),
    ]



from enum import IntEnum

from django.core.management import call_command
from django.core.management.base import BaseCommand

from marketmanager import __title__

# Eve Category IDs
MATERIAL = 4
SHIP = 6
MODULE = 7
CHARGE = 8
BLUEPRINT = 9
SKILLS  = 16
COMMODITY = 17
DRONE = 18
IMPLANT = 20
ASTERIOD = 25
APPAREL = 30
PLANETARY_INDUSTRY = 41
PLANETARY_COMMODITIES = 43
SPECIAL_EDITION_ASSETS = 63
STRUCTURE_MODULE = 66
SKINS = 91

class Command(BaseCommand):
    help = "Preloads data required for this app from ESI"

    def handle(self, *args, **options):
        call_command(
            "eveuniverse_load_types",
            __title__,
            "--category_id",
            str(MATERIAL),
            "--category_id",
            str(SHIP),
            "--category_id",
            str(MODULE),
            "--category_id",
            str(CHARGE),
            "--category_id",
            str(BLUEPRINT),
            "--category_id",
            str(SKILLS),
            "--category_id",
            str(COMMODITY),
            "--category_id",
            str(DRONE),
            "--category_id",
            str(IMPLANT),
            "--category_id",
            str(APPAREL),
            "--category_id",
            str(PLANETARY_INDUSTRY),
            "--category_id",
            str(PLANETARY_COMMODITIES),
            "--category_id",
            str(SPECIAL_EDITION_ASSETS),
            "--category_id",
            str(STRUCTURE_MODULE),
            "--category_id",
            str(SKINS),
        )



from unittest.mock import Mock

from eveuniverse.models import EveRegion, EveSolarSystem, EveType

from django.test import TestCase
from django.test.testcases import TestCase

from allianceauth.authentication.models import (
    EveAllianceInfo, EveCharacter, EveCorporationInfo, User,
)

from marketmanager.models import Order
from marketmanager.tasks import (
    fetch_all_character_orders, fetch_all_corporation_orders,
    fetch_all_corporations_structures, fetch_characters_character_id_orders,
    fetch_corporations_corporation_id_orders,
    fetch_corporations_corporation_id_structures,
    fetch_markets_region_id_orders, fetch_markets_structures_structure_id,
    fetch_public_market_orders, fetch_public_structures,
    fetch_universe_structures_structure_id,
)

from .testdata.load_eveuniverse import load_eveuniverse


def create_testdata():
    load_eveuniverse()
    EveCharacter.objects.all().delete()
    EveCharacter.objects.create(
        character_id=1,
        character_name='Character1',
        corporation_id=1,
        corporation_name='test corp',
        corporation_ticker='TEST'
        )
    EveCharacter.objects.create(
        character_id=2,
        character_name='Character2',
        corporation_id=1,
        corporation_name='test corp',
        corporation_ticker='TEST'
        )
    Order.objects.create(
        #expired order
        eve_type = EveType.objects.get(id = 44992)
        )

class TestMarketmanagerTasks(TestCase):
    def test_garbage_collection(self):
    # Orders
    # Generic Expiry Date Passed

    # Calculated Expiry Date Passed

    # State = Expired or Cancelled

    # Stale

    # Structures
    # Stale
        self.assertTrue(True)



import json
from unittest import mock

from django.test import TestCase

from marketmanager.models import Order, Structure
from marketmanager.providers import (
    get_characters_character_id_orders, get_corporations_corporation_id_orders,
    get_corporations_corporation_id_structures,
    get_markets_region_id_orders_paged, get_markets_structures_structure_id,
    get_universe_structures, get_universe_structures_structure_id,
)



import datetime

from celery import shared_task
from eveuniverse.models import EveRegion, EveSolarSystem, EveType

from django.core.exceptions import ObjectDoesNotExist
from django.db.models import Q, Sum

from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
from allianceauth.services.hooks import get_extension_logger
from esi.models import Token

from marketmanager.app_settings import (
    MARKETMANAGER_CLEANUP_DAYS_ORDER, MARKETMANAGER_CLEANUP_DAYS_STRUCTURE,
    MARKETMANAGER_TASK_PRIORITY_BACKGROUND, MARKETMANAGER_TASK_PRIORITY_ORDERS,
    MARKETMANAGER_TASK_PRIORITY_STRUCTURES,
    MARKETMANAGER_TASK_PRIORITY_WATCH_CONFIGS,
    MARKETMANAGER_TYPESTATISTICS_MINIMUM_ORDER_COUNT,
    MARKETMANAGER_WEBHOOK_COLOUR_ERROR, MARKETMANAGER_WEBHOOK_COLOUR_SUCCESS,
    MARKETMANAGER_WEBHOOK_COLOUR_WARNING,
)
from marketmanager.models import (
    Order, PublicConfig, StatisticsConfig, Structure, TypeStatistics,
    WatchConfig,
)
from marketmanager.providers import (
    get_characters_character_id_orders, get_corporations_corporation_id_orders,
    get_corporations_corporation_id_structures,
    get_markets_region_id_orders_paged, get_markets_structures_structure_id,
    get_universe_structures, get_universe_structures_structure_id,
)
from marketmanager.task_helpers import (
    create_embed_supply, fifth_percentile, get_corp_token,
    get_matching_privateconfig_token, get_random_market_token,
    is_existing_order, median, weighted_average,
)

from . import __version__

logger = get_extension_logger(__name__)


@shared_task
def fetch_public_market_orders():
    """Fetch&Save Public Market Orders for configured regions
    bulk calls fetch_markets_region_id_orders(region_id: int)"""
    logger.debug("fetch_public_market_orders(), Fetching configured regions")
    for region in PublicConfig.objects.get(id=1).fetch_regions.all():
        logger.debug(f"fetch_public_market_orders(), Queuing up celery task for region {region.id}")
        fetch_markets_region_id_orders.apply_async(args=[region.id, 'buy'], priority=MARKETMANAGER_TASK_PRIORITY_ORDERS)
        fetch_markets_region_id_orders.apply_async(args=[region.id, 'sell'], priority=MARKETMANAGER_TASK_PRIORITY_ORDERS)


@shared_task
def fetch_markets_region_id_orders(region_id: int, order_type: str = "all"):
    logger.debug(f"fetch_markets_region_id_orders({region_id})")
    order_eve_region = EveRegion.objects.get(id=region_id)
    if order_type == 'buy':
        current_orders = Order.objects.filter(
            eve_region=order_eve_region,
            is_buy_order = True,
        )
    elif order_type == 'sell':
        current_orders = Order.objects.filter(
            eve_region=order_eve_region,
            is_buy_order = False,
        )
    else:
        current_orders = Order.objects.filter(
        eve_region=order_eve_region,
        )

    current_page = 1
    total_pages = 1
    while current_page <= total_pages:
        new_orders = []
        updated_orders = []
        order_page, order_page_headers = get_markets_region_id_orders_paged(region_id, current_page, order_type)
        total_pages = int(order_page_headers.headers['X-Pages'])
        current_page += 1
        for order in order_page:
            existing_order = is_existing_order(order, current_orders)
            if existing_order is not False:
                if existing_order.price != order["price"] or existing_order.volume_remain != order["volume_remain"]:
                    existing_order.price = order["price"]
                    existing_order.volume_remain = order["volume_remain"]
                    updated_orders.append(existing_order)
                else:
                    # It is an existing order, but hasnt changed, so nothing
                    pass
            else:
                order_eve_type, order_eve_type_fetched = EveType.objects.get_or_create_esi(
                    id=order["type_id"],
                    enabled_sections=[EveType.Section.MARKET_GROUPS]
                )
                new_order = Order(order_id=order["order_id"])
                new_order.eve_type = order_eve_type
                new_order.duration = order["duration"]
                new_order.is_buy_order = order["is_buy_order"]
                new_order.issued = order["issued"]
                new_order.location_id = order["location_id"]
                new_order.min_volume = order["min_volume"]
                new_order.price = order["price"]
                new_order.range = order["range"]
                new_order.eve_solar_system_id=order["system_id"]
                new_order.eve_region = order_eve_region
                new_order.volume_remain = order["volume_remain"]
                new_orders.append(new_order)
        try:
            Order.objects.bulk_create(new_orders, batch_size=500)
            Order.objects.bulk_update(updated_orders, batch_size=500, fields=['price', 'volume_remain'])
        except Exception as e:
            logger.error(e)


@shared_task
def fetch_all_character_orders():
    """Fetch&Save every Characters Market Orders
    bulk calls fetch_characters_character_id_orders(character_id)"""
    logger.debug("fetch_all_character_orders()")
    character_ids = Token.objects.values_list('character_id').require_scopes(
        ["esi-markets.read_character_orders.v1"])
    unique_character_ids = list(dict.fromkeys(character_ids))
    for character_id in unique_character_ids:
        logger.debug(f"fetch_all_character_orders(), Queuing up celery task for character {character_id}")
        fetch_characters_character_id_orders.apply_async(args=[character_id[0]], priority=MARKETMANAGER_TASK_PRIORITY_ORDERS)



@shared_task
def fetch_characters_character_id_orders(character_id: int, order_type: str = "all"):
    """Fetch&Save a single Characters Market Orders
    bulk called by fetch_all_character_orders()

    Parameters
    ----------
    corporation_id: int
        Should match a valid Character ID"""
    logger.debug(f"fetch_characters_character_id_orders({character_id})")
    for order in get_characters_character_id_orders(character_id):

        order_eve_type, order_eve_type_fetched = EveType.objects.get_or_create_esi(
            id=order["type_id"],
            enabled_sections=[EveType.Section.MARKET_GROUPS]
        )
        order_eve_region, order_eve_region_fetched = EveRegion.objects.get_or_create_esi(
            id=order["region_id"]
        )
        try:
            order_eve_character = EveCharacter.objects.get(
                character_id=character_id
            )
        except ObjectDoesNotExist:
            EveCharacter.objects.create_character(character_id)
            order_eve_character = EveCharacter.objects.get(
                character_id=character_id
            )

        try:
            Order.objects.update_or_create(
                order_id=order["order_id"],
                defaults={
                    'eve_type': order_eve_type,
                    'duration': order["duration"],
                    'is_buy_order': order.get("is_buy_order", False),
                    'is_corporation': order["is_corporation"],
                    'issued': order["issued"],
                    'issued_by_character': order_eve_character,
                    'location_id': order["location_id"],
                    'eve_region': order_eve_region,
                    'min_volume': order["min_volume"],
                    'price': order["price"],
                    'range': order["range"],
                    'volume_remain': order["volume_remain"],
                    }
                )
        except Exception as e:
            logger.error(e)
        logger.debug(f"fetch_characters_character_id_orders({character_id}): Saved Order {order_eve_type.name}")


@shared_task
def fetch_all_corporation_orders():
    """Fetch&Save every Corporations Market Orders
    bulk calls fetch_corporations_corporation_id_orders(corporation_id)"""
    logger.debug("fetch_all_corporation_orders()")
    for corporation in EveCorporationInfo.objects.all():
        logger.debug(f"fetch_all_corporation_orders(), Queuing up celery task for corporation {corporation.id}")
        fetch_corporations_corporation_id_orders.apply_async(args=[corporation.corporation_id], priority=MARKETMANAGER_TASK_PRIORITY_ORDERS)



@shared_task
def fetch_corporations_corporation_id_orders(corporation_id: int, order_type: str = "all"):
    """Fetch&Save a Corporations Market Orders
    Is Bulk-Called by fetch_all_corporation_orders()

    Parameters
    ----------
    corporation_id: int
        Should match a valid Corporation ID"""
    logger.debug(f"fetch_corporations_corporation_id_orders({corporation_id})")
    scopes = ["esi-markets.read_corporation_orders.v1"]
    req_roles = ["Accountant", "Trader"]

    token = get_corp_token(corporation_id, scopes, req_roles)
    if token is False:
        logger.error(f"No Token for Corporation {corporation_id}")
        return
    order_eve_corporation = EveCorporationInfo.objects.get(
        corporation_id=corporation_id
    )

    for order in get_corporations_corporation_id_orders(corporation_id, token):

        order_eve_type, order_eve_type_fetched = EveType.objects.get_or_create_esi(
            id=order["type_id"],
            enabled_sections=[EveType.Section.MARKET_GROUPS]
        )
        order_eve_region, order_eve_region_fetched = EveRegion.objects.get_or_create_esi(
            id=order["region_id"]
        )
        try:
            order_eve_character = EveCharacter.objects.get(
                character_id=order["issued_by"]
            )
        except ObjectDoesNotExist:
            EveCharacter.objects.create_character(order["issued_by"])
            order_eve_character = EveCharacter.objects.get(
                character_id=order["issued_by"]
            )
        try:
            Order.objects.update_or_create(
                order_id=order["order_id"],
                defaults={
                    'eve_type': order_eve_type,
                    'duration': order["duration"],
                    'is_buy_order': order.get("is_buy_order", False),
                    'is_corporation': True,
                    'issued': order["issued"],
                    'issued_by_character': order_eve_character,
                    'issued_by_corporation': order_eve_corporation,
                    'wallet_division': order["wallet_division"],
                    'location_id': order["location_id"],
                    'eve_region': order_eve_region,
                    'min_volume': order["min_volume"],
                    'price': order["price"],
                    'range': order["range"],
                    'volume_remain': order["volume_remain"],
                    }
            )
        except Exception as e:
            logger.error(e)


@shared_task()
def fetch_public_structures():
    logger.debug("fetch_public_structures()")
    for structure_id in get_universe_structures(filter="market"):
        logger.debug(f"fetch_public_structures(), Queuing up celery task for structure {structure_id}")
        fetch_universe_structures_structure_id.apply_async(args=[structure_id], priority=MARKETMANAGER_TASK_PRIORITY_STRUCTURES)

@shared_task()
def update_private_structures():
    logger.debug("update_private_structures()")
    for structure in Structure.objects.all():
        fetch_universe_structures_structure_id(structure.structure_id, public = False)

@shared_task()
def fetch_universe_structures_structure_id(structure_id: int, public = True):
    logger.debug(f"fetch_universe_structures_structure_id({structure_id})")
    if public == True:
        token = get_random_market_token()
    else:
        token = get_matching_privateconfig_token(structure_id)

    if token is False:
        logger.error(f"No Public or Private token (Public={public}) for {structure_id}")
        return

    try:
        structure = get_universe_structures_structure_id(
            structure_id,
            token
        )
    except Exception as e:
        logger.error(e)
        return

    structure_eve_solar_system, structure_eve_solar_system_fetched = EveSolarSystem.objects.get_or_create_esi(
        id=structure["solar_system_id"]
    )
    structure_eve_type, structure_eve_type_fetched = EveType.objects.get_or_create_esi(
        id=structure["type_id"],
        enabled_sections=[EveType.Section.MARKET_GROUPS]
    )
    try:
        Structure.objects.update_or_create(
            structure_id=structure_id,
            defaults={
                'name': structure["name"],
                'owner_id': structure["owner_id"],
                'solar_system': structure_eve_solar_system,
                'eve_type': structure_eve_type
            }
        )
    except Exception as e:
        logger.error(e)

@shared_task()
def fetch_all_structure_orders():
    logger.debug("fetch_all_structure_orders()")
    for structure in Structure.objects.all():
        logger.debug(f"fetch_all_structure_orders(), Queuing up celery task for structure {structure.structure_id}")
        fetch_markets_structures_structure_id.apply_async(args=[structure.structure_id], priority=MARKETMANAGER_TASK_PRIORITY_STRUCTURES)


@shared_task()
def fetch_markets_structures_structure_id(structure_id: int):
    logger.debug(f"fetch_markets_structures_structure_id({structure_id})")

    token = get_matching_privateconfig_token(structure_id)

    if token is False:
        logger.error(f"No Token PrivateConfig for structure {structure_id}")
        return

    order_eve_region = Structure.objects.get(structure_id = structure_id).solar_system.eve_constellation.eve_region
    order_eve_solar_system = Structure.objects.get(structure_id = structure_id).solar_system

    for order in get_markets_structures_structure_id(structure_id, token):

        order_eve_type, order_eve_type_fetched = EveType.objects.get_or_create_esi(
            id=order["type_id"],
            enabled_sections=[EveType.Section.MARKET_GROUPS]
        )

        try:
            Order.objects.update_or_create(
                order_id=order["order_id"],
                defaults={
                    'eve_type': order_eve_type,
                    'duration': order["duration"],
                    'is_buy_order': order["is_buy_order"],
                    'issued': order["issued"],
                    'location_id': order["location_id"],
                    'eve_solar_system': order_eve_solar_system,
                    'eve_region': order_eve_region,
                    'min_volume': order["min_volume"],
                    'price': order["price"],
                    'range': order["range"],
                    'volume_remain': order["volume_remain"],
                    }
            )
        except Exception as e:
            logger.error(e)

@shared_task()
def fetch_all_corporations_structures():
    logger.debug("fetch_all_corporations_structures()")
    for corporation in EveCorporationInfo.objects.all():
        logger.debug(f"fetch_all_corporations_structures(), Queuing up celery task for corporation {corporation.corporation_id}")
        fetch_corporations_corporation_id_structures.apply_async(args=[corporation.corporation_id], priority=MARKETMANAGER_TASK_PRIORITY_STRUCTURES)



@shared_task()
def fetch_corporations_corporation_id_structures(corporation_id: int):
    logger.debug(f"fetch_corporations_corporation_id_structures({corporation_id})")
    scopes = ["esi-corporations.read_structures.v1"]
    req_roles = ["Station_Manager"]

    token = get_corp_token(corporation_id, scopes, req_roles)
    if token is False:
        logger.error(f"No Token for Corporation {corporation_id}")
        return

    for structure in get_corporations_corporation_id_structures(corporation_id, token):
        for service in structure['services']:
            if service['name'] == "market":
                structure_eve_solar_system, structure_eve_solar_system_fetched = EveSolarSystem.objects.get_or_create_esi(
                    id=structure["solar_system_id"]
                )
                structure_eve_type, structure_eve_type_fetched = EveType.objects.get_or_create_esi(
                    id=structure["type_id"],
                    enabled_sections=[EveType.Section.MARKET_GROUPS]
                )
                try:
                    Structure.objects.update_or_create(
                        structure_id=structure["structure_id"],
                        defaults={
                            'name': structure["name"],
                            'owner_id': structure["corporation_id"],
                            'solar_system': structure_eve_solar_system,
                            'eve_type': structure_eve_type
                        }
                    )
                except Exception as e:
                    logger.error(e)

@shared_task()
def run_all_watch_configs():
    for config in WatchConfig.objects.all():
        if config.config_type == "Supply":
            run_watch_config_supply.apply_async(args=[config.id], priority=MARKETMANAGER_TASK_PRIORITY_WATCH_CONFIGS)
        elif config.config_type == "Bargain":
            pass
        elif config.config_type == "Scalp":
            pass

@shared_task()
def run_watch_config_supply(id: int):
    config = WatchConfig.objects.get(id=id)

    # Initial order selection
    matching_orders = Order.objects.filter(
        eve_type = config.eve_type,
        is_buy_order = config.buy_order)

    # Optional Location Filters
    if config.structure.count() > 0 or config.solar_system.count() > 0 or config.region.count() > 0:
        matching_orders_location = matching_orders.filter(
            Q(location_id__in=config.structure.all()) | Q(eve_solar_system__in=config.solar_system.all()) | Q(eve_region__in=config.region.all()))
    else:
        matching_orders_location = matching_orders

    if config.structure_type.count() > 0 :
        valid_structures = Structure.objects.filter(eve_type__in=config.structure_type.all())
        matching_orders_location = matching_orders_location.filter(location_id__in=valid_structures.all())

    # Comparators
    ## if price is not zero, filter out all orders that dont meet minimum
    if config.price != 0 and config.buy_order == True:
        matching_orders_location_price = matching_orders_location.filter(price__gte=config.price)
    elif config.price != 0 and config.buy_order == False:
        matching_orders_location_price = matching_orders_location.filter(price__lte=config.price)
    else:
        matching_orders_location_price = matching_orders_location

    matched_volume = matching_orders_location_price.aggregate(volume=Sum('volume_remain'))["volume"]
    if matching_orders_location_price.count() > 0:
        if matched_volume >= config.volume:
            embed = create_embed_supply(
                config=config,
                matched_volume=matched_volume,
                description = "The following Supply Check reported no Errors",
                buy_order=config.buy_order,
                colour = MARKETMANAGER_WEBHOOK_COLOUR_SUCCESS)
            webhooks = config.debug_webhooks.all()
            for webhook in webhooks:
                if webhook.enabled:
                    webhook.send_embed(embed)

        else:
            embed = create_embed_supply(
                config=config,
                matched_volume=matched_volume,
                description = "The following Supply Check failed to return enough Volume",
                buy_order=config.buy_order,
                colour = MARKETMANAGER_WEBHOOK_COLOUR_WARNING)
            webhooks = config.webhooks.all()
            for webhook in webhooks:
                if webhook.enabled:
                    webhook.send_embed(embed)
    else:
        embed = create_embed_supply(
            config=config,
            matched_volume=0,
            description = "The following Supply Check returned zero volume, This item may be out of stock or Market Manager may be incorrectly configured",
            buy_order=config.buy_order,
            colour = MARKETMANAGER_WEBHOOK_COLOUR_ERROR)
        webhooks = config.webhooks.all()
        for webhook in webhooks:
            if webhook.enabled:
                webhook.send_embed(embed)


@shared_task()
def garbage_collection():
    # Orders
    # Generic Expiry Date Passed
    order_passed_expiry_date = datetime.datetime.now() - datetime.timedelta(days = 90)
    orders_expired_by_generic_date = Order.objects.filter(issued__lt=order_passed_expiry_date)
    orders_expired_by_generic_date.delete()
    # Calculated Expiry Date Passed
    # Cant do this, property
    # orders_expired_by_calculated_date = Order.objects.filter(expiry__lt=datetime.datetime.now())
    # orders_expired_by_calculated_date.delete()
    # State = Expired or Cancelled
    orders_expired_by_state = Order.objects.filter(Q(state="Expired") | Q(state="Cancelled"))
    orders_expired_by_state.delete()
    # Stale
    order_stale_cleanup_date = datetime.datetime.now() - datetime.timedelta(days = MARKETMANAGER_CLEANUP_DAYS_ORDER)
    orders_stale = Structure.objects.filter(updated_at__lt=order_stale_cleanup_date)
    orders_stale.delete()
    # Structures
    # Stale
    structure_stale_cleanup_date = datetime.datetime.now() - datetime.timedelta(days = MARKETMANAGER_CLEANUP_DAYS_STRUCTURE)
    structures_stale = Structure.objects.filter(updated_at__lt=structure_stale_cleanup_date)
    structures_stale.delete()

@shared_task()
def update_all_type_statistics():
    logger.debug(f"update_all_type_statistics()")
    for eve_type in EveType.objects.filter(eve_market_group__isnull=False):
        if Order.objects.filter(eve_type = eve_type).count() >= MARKETMANAGER_TYPESTATISTICS_MINIMUM_ORDER_COUNT:
            calculate_type_statistics.apply_async(args=[eve_type.id], priority=MARKETMANAGER_TASK_PRIORITY_BACKGROUND)
        else:
            TypeStatistics.objects.filter(eve_type = eve_type).delete()

@shared_task
def calculate_type_statistics(type_id: int):
    eve_type = EveType.objects.get(id = type_id)
    try:
        logger.debug(f"calculate_type_statistics EveType ({eve_type.name}) Region (NEW EDEN)")
        type_statistic = TypeStatistics.objects.update_or_create(
            eve_type = eve_type,
            eve_region = None,
            defaults = {
                'buy_fifth_percentile' : fifth_percentile(eve_type = eve_type, buy_order = True),
                'sell_fifth_percentile': fifth_percentile(eve_type = eve_type, buy_order = False),
                'buy_weighted_average': weighted_average(eve_type = eve_type, buy_order = True),
                'sell_weighted_average': weighted_average(eve_type = eve_type, buy_order = False),
                'buy_median': median(eve_type = eve_type, buy_order = True),
                'sell_median': median(eve_type = eve_type, buy_order = False),
            }
        )
    except Exception as e:
        logger.error(e)

    for eve_region in StatisticsConfig.get_solo().calculate_regions.all():
        logger.debug(f"calculate_type_statistics EveType ({eve_type.name}) Region ({eve_region.name})")
        if Order.objects.filter(eve_type = eve_type, eve_region = eve_region).exists():
            try:
                type_statistic = TypeStatistics.objects.update_or_create(
                    eve_type = eve_type,
                    eve_region = eve_region,
                    defaults = {
                        'buy_fifth_percentile' : fifth_percentile(eve_type = eve_type, eve_region = eve_region, buy_order = True),
                        'sell_fifth_percentile': fifth_percentile(eve_type = eve_type, eve_region = eve_region, buy_order = False),
                        'buy_weighted_average': weighted_average(eve_type = eve_type, eve_region = eve_region, buy_order = True),
                        'sell_weighted_average': weighted_average(eve_type = eve_type, eve_region = eve_region, buy_order = False),
                        'buy_median': median(eve_type = eve_type, eve_region = eve_region, buy_order = True),
                        'sell_median': median(eve_type = eve_type, eve_region = eve_region, buy_order = False),
                    }
                )
            except Exception as e:
                logger.error(e)
        else: # Incase we have old statistics, we'd want to wipe them for objects no longer on the market
            try:
                type_statistic = TypeStatistics.objects.get(eve_type = eve_type, eve_region = eve_region).delete()
            except TypeStatistics.DoesNotExist:
                pass
            except Exception as e:
                logger.error(e)



# Generated by Django 4.0.3 on 2022-03-19 11:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eveuniverse', '0005_type_materials_and_sections'),
        ('marketmanager', '0024_remove_typestatistics_new_eden'),
    ]

    operations = [
        migrations.CreateModel(
            name='StatisticsConfig',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('calculate_regions', models.ManyToManyField(to='eveuniverse.everegion', verbose_name='TypeStatistics Calculation Regions')),
            ],
            options={
                'default_permissions': (),
            },
        ),
    ]



# Generated by Django 4.0.3 on 2022-03-19 04:03

from decimal import Decimal

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketmanager', '0019_remove_order_escrow_remove_order_source_character_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='order',
            name='duration',
            field=models.PositiveSmallIntegerField(help_text='Number of days the order was valid for (starting from the issued date). An order expires at time issued + duration', verbose_name='Duration'),
        ),
        migrations.AlterField(
            model_name='order',
            name='price',
            field=models.DecimalField(decimal_places=2, help_text='Cost per unit for this order', max_digits=15, verbose_name='Price'),
        ),
        migrations.AlterField(
            model_name='order',
            name='range',
            field=models.CharField(choices=[('1', '1'), ('10', '10'), ('2', '2'), ('20', '20'), ('3', '3'), ('30', '30'), ('4', '4'), ('40', '40'), ('5', '5'), ('region', 'Region'), ('solarsystem', 'Solar System'), ('station', 'Station')], help_text='Valid order range, numbers are ranges in jumps', max_length=12, verbose_name='Order Range'),
        ),
        migrations.AlterField(
            model_name='order',
            name='state',
            field=models.CharField(choices=[('', ''), ('cancelled', 'Cancelled'), ('expired ', 'Expired')], help_text='Current order state, Only valid for Authenticated order History. Will not update from Public Market Data.', max_length=10, verbose_name='Order State'),
        ),
        migrations.AlterField(
            model_name='structure',
            name='name',
            field=models.CharField(max_length=60, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='typestatistics',
            name='buy_fifth_percentile',
            field=models.DecimalField(decimal_places=2, max_digits=15, verbose_name='Buy 5th Percentile'),
        ),
        migrations.AlterField(
            model_name='typestatistics',
            name='buy_median',
            field=models.DecimalField(decimal_places=2, max_digits=15, verbose_name='Buy Median'),
        ),
        migrations.AlterField(
            model_name='typestatistics',
            name='buy_weighted_average',
            field=models.DecimalField(decimal_places=2, max_digits=15, verbose_name='Buy Weighted Average'),
        ),
        migrations.AlterField(
            model_name='typestatistics',
            name='sell_fifth_percentile',
            field=models.DecimalField(decimal_places=2, max_digits=15, verbose_name='Sell 5th Percentile'),
        ),
        migrations.AlterField(
            model_name='typestatistics',
            name='sell_median',
            field=models.DecimalField(decimal_places=2, max_digits=15, verbose_name='Sell Median'),
        ),
        migrations.AlterField(
            model_name='typestatistics',
            name='sell_weighted_average',
            field=models.DecimalField(decimal_places=2, max_digits=15, verbose_name='Sell Weighted Average'),
        ),
        migrations.AlterField(
            model_name='watchconfig',
            name='price',
            field=models.DecimalField(decimal_places=2, default=Decimal('0'), help_text='Set to Zero to skip this filter', max_digits=15, verbose_name='Price'),
        ),
    ]



# Generated by Django 4.0.3 on 2022-03-15 05:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketmanager', '0009_remove_watchconfig_fail_webhooks_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='watchconfig',
            name='debug_webhooks',
            field=models.ManyToManyField(blank=True, help_text='Primarily for Testing/Debugging Purposes. This webhook will receive updates on WatchConfigs that _dont_ notify. Because their Configs didnt Meet/Breach.', related_name='debug_webhooks', to='marketmanager.webhook', verbose_name='Debug Webhook'),
        ),
    ]



# Generated by Django 3.2.8 on 2021-11-29 13:51

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('marketmanager', '0002_auto_20211129_1339'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='general',
            options={'default_permissions': (), 'managed': False, 'permissions': (('basic_market_browser', 'Can access the Market Browser with standard features'), ('advanced_market_browser', 'Can access the Market Browser with more detail such as the owners of orders'))},
        ),
    ]



import os

from setuptools import find_packages, setup

from marketmanager import __version__

# read the contents of your README file
this_directory = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name="aa-market-manager",
    version=__version__,
    packages=find_packages(),
    include_package_data=True,
    license="MIT",
    description="AllianceAuth Market Management Tool",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Joel Falknau',
    author_email='ozirascal@gmail.com',
    url="https://gitlab.com/tactical-supremacy/aa-market-manager",
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 3.2',
        'Framework :: Django :: 4.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',

    ],
    python_requires="~=3.8",
    install_requires=[
        "allianceauth>=2.11.0",
        "django-eveuniverse",
        "py-cord==2.0.0b1",
        "django-solo>=2.0.0"
        ],
)



import datetime
from decimal import Decimal

from discord import SyncWebhook
from eveuniverse.models import EveRegion, EveSolarSystem, EveType
from solo.models import SingletonModel

from django.core.exceptions import ValidationError
from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from django.utils.translation import gettext_lazy as _

from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
from esi.models import Token


class General(models.Model):
    """Meta model for app permissions"""

    class Meta:
        managed = False
        default_permissions = ()
        permissions = (
            ("basic_market_browser", "Can access the Standard Market Browser"),
            ("order_highlight_user", "Market orders owned by the user's characters may be highlighted in the standard/basic Market Browser"),
            ("order_highlight_corporation", "Market orders owned by any corporation a user is a member of may be highlighted in the standard/basic Market Browser WARNING: This has no checks for Corporation Roles."),
            ("advanced_market_browser", "Can access the Avanced Market Browser"),
            ("can_add_token_character", "Can add a Character Token with required scopes"),
            ("can_add_token_corporation", "Can add a Corpration Token with required scopes"),
            )


class Order(models.Model):
    """An EVE Market Order"""
    order_id = models.PositiveBigIntegerField(
        _("Order ID"),
        help_text="Unique order ID",
        primary_key=True)
    eve_type = models.ForeignKey(
        EveType,
        verbose_name=_("Type"),
        on_delete=models.CASCADE)
    duration = models.PositiveSmallIntegerField(
        _("Duration"),
        help_text="Number of days the order was valid for (starting from the issued date). An order expires at time issued + duration")
    is_buy_order = models.BooleanField(
        _("Buy Order"),
        default=False,
        help_text="True if the order is a bid (buy) order",
        db_index=True)
    issued = models.DateTimeField(
        _("Issued"),
        help_text="Date and time when this order was issued",
        auto_now=False,
        auto_now_add=False)
    location_id = models.PositiveBigIntegerField(
        _("Location ID"),
        help_text="ID of the location where order was placed")
    eve_solar_system = models.ForeignKey(
        EveSolarSystem,
        verbose_name=_("System"),
        on_delete=models.CASCADE,
        null=True,
        blank=True,)
    eve_region = models.ForeignKey(
        EveRegion,
        verbose_name=_("Region"),
        on_delete=models.CASCADE)
    min_volume = models.PositiveIntegerField(
        _("Minimum Volume"),
        null=True,
        blank=True,
        help_text="For buy orders, the minimum quantity that will be accepted in a matching sell order")
    price = models.DecimalField(
        _("Price"),
        max_digits=15,
        decimal_places=2,
        help_text="Cost per unit for this order")
    RANGE_CHOICES = [
        ('1', '1'), ('10', '10'), ('2', '2'), ('20', '20'), ('3', '3'),
        ('30', '30'), ('4', '4'), ('40', '40'), ('5', '5'),
        ('region', _('Region')),
        ('solarsystem', _('Solar System')),
        ('station', _('Station'))]
    range = models.CharField(
        _("Order Range"),
        max_length=12,
        choices=RANGE_CHOICES,
        help_text="Valid order range, numbers are ranges in jumps")
    volume_remain = models.PositiveIntegerField(
        _("Volume Remaining"),
        help_text="Quantity of items still required or offered")
    is_corporation = models.BooleanField(
        _("Is Corporation"),
        default=False,
        help_text="Signifies whether the buy/sell order was placed on behalf of a corporation.")
    wallet_division = models.PositiveSmallIntegerField(
        _("Wallet Division"),
        null=True,
        blank=True,
        help_text="The corporation wallet division used for this order.")
    issued_by_character = models.ForeignKey(
        EveCharacter,
        verbose_name=_("Character"),
        on_delete=models.SET_NULL,
        null=True,
        blank=True)
    issued_by_corporation = models.ForeignKey(
        EveCorporationInfo,
        verbose_name=_("Corporation"),
        on_delete=models.SET_NULL,
        null=True,
        blank=True,)
    STATE_CHOICES = [
        ('', ''),
        ('cancelled', _('Cancelled')),
        ('expired ', _('Expired'))]
    state = models.CharField(
        _("Order State"),
        max_length=10,
        choices=STATE_CHOICES,
        help_text="Current order state, Only valid for Authenticated order History. Will not update from Public Market Data.")
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        """
        Meta definitions
        """
        default_permissions = ()

    @property
    def expiry(self):
        return self.issued + datetime.timedelta(days=self.duration)


class Structure(models.Model):
    """An EVE Online Upwell Structure"""
    structure_id = models.PositiveBigIntegerField(
        _("Structure ID"),
        primary_key=True)
    name = models.CharField(
        _("Name"),
        max_length=60)
    owner_id = models.IntegerField(_("Owner Corporation ID"))
    solar_system = models.ForeignKey(
        EveSolarSystem,
        verbose_name=_("Solar System"),
        on_delete=models.CASCADE)
    eve_type = models.ForeignKey(
        EveType,
        verbose_name=_("Type"),
        on_delete=models.CASCADE)
    pull_market = models.BooleanField(
        _("Pull Market Orders"),
        help_text="Useful to ignore specific structures for _reasons_",
        default=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.name
    class Meta:
        """
        Meta definitions
        """
        default_permissions = ()



class Webhook(models.Model):
    name = models.CharField(
        _("Name"),
        max_length=100)
    url = models.URLField(
        _("URL"),
        max_length=200)
    enabled = models.BooleanField(default=True)

    def __str__(self):
        return self.name

    def send_embed(self, embed):
        webhook = SyncWebhook.from_url(self.url)
        webhook.send(embed=embed, username="Market Manager")

    class Meta:
        """
        Meta definitions
        """
        default_permissions = ()


class WatchConfig(models.Model):
    """Rules to Watch"""
    buy_order = models.BooleanField(_("Buy Order"))
    eve_type = models.ForeignKey(
        EveType,
        verbose_name=_("EVE Type"),
        on_delete=models.CASCADE)
    # Location Rules, Combined
    structure = models.ManyToManyField(
        Structure,
        verbose_name=_("Structure"),
        blank=True)
    solar_system = models.ManyToManyField(
        EveSolarSystem,
        verbose_name=_("Solar System"),
        blank=True)
    region = models.ManyToManyField(
        EveRegion,
        verbose_name=_("Region"),
        blank=True)
    # Filter
    structure_type = models.ManyToManyField(
        EveType,
        help_text="Filter by structure Type/Size/Docking (ie, forts/keeps for cap fuel)",
        verbose_name=_("Structure Type Filter"),
        related_name="structure_type",
        blank=True)
    # Settings
    volume = models.IntegerField(
        _("Volume"),
        help_text="Set to Zero to check ANY/EVERY order against Price",
        default=1,
        blank=False)
    price = models.DecimalField(
        _("Price"),
        help_text="Set to Zero to skip this filter",
        default=Decimal(0),
        max_digits=15,
        decimal_places=2,
        blank=False)
    jita_compare_percent = models.DecimalField(
        _("Jita Comparison %"),
        help_text="If set ignores Flat price value",
        max_digits=3,
        decimal_places=0,
        default=Decimal(0),
        validators=[MinValueValidator(0), MaxValueValidator(100)])
    CONFIG_TYPE_CHOICES = [
        ("Supply", _("Supply Check")),
        ("Bargain", _("Bargain Finder")),
        ("Scalp", _("Scalp Finder"))]
    config_type = models.CharField(
        _("Config Type"),
        max_length=10,
        choices=CONFIG_TYPE_CHOICES,
        help_text="The type of WatchConfig, check documentation for more.",
        blank=False,
        default="Supply")

    # Type
        # Supply Check, Ensure there is available Volume at the Given Price with conditions
        # Bargain Finder, Highlight orders that fall under the given thresholds.
        # Scalp Finder, Highlight orders that breach conditions.

    # Destinations

    webhooks = models.ManyToManyField(
        Webhook,
        verbose_name=_("Webhooks"),
        blank=True)
    debug_webhooks = models.ManyToManyField(
        Webhook,
        verbose_name=_("Debug Webhook"),
        related_name="debug_webhooks",
        blank=True,
        help_text="Primarily for Testing/Debugging Purposes. This webhook will receive updates on WatchConfigs that _dont_ notify. Because their Configs didnt Meet/Breach.")

    managed_app = models.CharField(
        _("Managed By App"),
        max_length=50,
        default="",
        help_text="The App managing this WatchConfig",
        blank=True,)
    managed_app_identifier = models.CharField(
        _("Managed App-Identifier"),
        max_length=50,
        default="",
        help_text="An identifier relevant to the App managing this WatchConfig",
        blank=True,)
    manged_app_reason = models.CharField(
        _("Managed App Reason"),
        max_length=50,
        default="",
        help_text="User Facing reason for this Managed WatchConfig",
        blank=True,)

    class Meta:
        """
        Meta definitions
        """
        default_permissions = ()


class PublicConfig(SingletonModel):
    fetch_regions = models.ManyToManyField(
        EveRegion,
        verbose_name=_("Fetch Regions")
    )

    def __str__(self):
        return "Public Market Configuration"

    class Meta:
        """
        Meta definitions
        """
        default_permissions = ()
        verbose_name = "Public Market Configuration"


class StatisticsConfig(SingletonModel):
    calculate_regions = models.ManyToManyField(
        EveRegion,
        verbose_name=_("TypeStatistics Calculation Regions")
    )


    def __str__(self):
        return "TypeStatistics Calculation Configuration"
    class Meta:
        """
        Meta definitions
        """
        default_permissions = ()
        verbose_name = "TypeStatistics Calculation Configuration"


class PrivateConfig(models.Model):
    token = models.OneToOneField(
        Token,
        verbose_name=_("ESI Token"),
        on_delete=models.CASCADE
        )
    valid_corporations = models.ManyToManyField(
        EveCorporationInfo,
        verbose_name=_("Valid Corporation Markets for this Token"),
        blank=True,
        )
    valid_structures = models.ManyToManyField(
        Structure,
        verbose_name=_("Valid Structure Markets for this Token"),
        blank=True,
        )
    failed = models.BooleanField(_("Disabled due to Failure, Check reason, adjust config and re-enable"))
    failure_reason = models.CharField(
        _("Failure Reason"),
        max_length=100,
        blank=True,
        default="")

    class Meta:
        """
        Meta definitions
        """
        default_permissions = ()

class TypeStatistics(models.Model):
    eve_type = models.ForeignKey(
        EveType,
        verbose_name=_(""),
        on_delete=models.CASCADE
        )
    eve_region = models.ForeignKey(
        EveRegion,
        verbose_name=_(""),
        on_delete=models.CASCADE,
        blank=True,
        null=True
        )
    buy_fifth_percentile = models.DecimalField(
        _("Buy 5th Percentile"),
        max_digits=15,
        decimal_places=2
        )
    sell_fifth_percentile = models.DecimalField(
        _("Sell 5th Percentile"),
        max_digits=15,
        decimal_places=2
        )
    buy_weighted_average = models.DecimalField(
        _("Buy Weighted Average"),
        max_digits=15,
        decimal_places=2
        )
    sell_weighted_average = models.DecimalField(
        _("Sell Weighted Average"),
        max_digits=15,
        decimal_places=2
        )
    buy_median = models.DecimalField(
        _("Buy Median"),
        max_digits=15,
        decimal_places=2
        )
    sell_median = models.DecimalField(
        _("Sell Median"),
        max_digits=15,
        decimal_places=2
        )

    class Meta:
        """
        Meta definitions
        """
        default_permissions = ()



# Generated by Django 4.0.3 on 2022-03-14 12:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eveuniverse', '0005_type_materials_and_sections'),
        ('marketmanager', '0004_alter_watchconfig_jita_compare_percent_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='watchconfig',
            name='constellation',
            field=models.ManyToManyField(blank=True, to='eveuniverse.eveconstellation', verbose_name='Constellation'),
        ),
        migrations.AlterField(
            model_name='watchconfig',
            name='location',
            field=models.ManyToManyField(blank=True, to='marketmanager.structure', verbose_name='Structure'),
        ),
        migrations.AlterField(
            model_name='watchconfig',
            name='region',
            field=models.ManyToManyField(blank=True, to='eveuniverse.everegion', verbose_name='Region'),
        ),
        migrations.AlterField(
            model_name='watchconfig',
            name='solar_system',
            field=models.ManyToManyField(blank=True, to='eveuniverse.evesolarsystem', verbose_name='Solar System'),
        ),
        migrations.AlterField(
            model_name='watchconfig',
            name='structure_type',
            field=models.ManyToManyField(blank=True, help_text='Filter by structure Type/Size/Docking (ie, forts/keeps for cap fuel)', related_name='structure_type', to='eveuniverse.evetype', verbose_name='Structure Type Filter'),
        ),
    ]



# Generated by Django 4.0.3 on 2022-03-18 04:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketmanager', '0017_alter_watchconfig_managed_app_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='typestatistics',
            name='buy_5_percent',
        ),
        migrations.RemoveField(
            model_name='typestatistics',
            name='sell_5_percent',
        ),
        migrations.AddField(
            model_name='typestatistics',
            name='buy_fifth_percentile',
            field=models.DecimalField(decimal_places=2, default=0, max_digits=20, verbose_name='Buy 5th Percentile'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='typestatistics',
            name='sell_fifth_percentile',
            field=models.DecimalField(decimal_places=2, default=0, max_digits=20, verbose_name='Sell 5th Percentile'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='typestatistics',
            name='buy_median',
            field=models.DecimalField(decimal_places=2, max_digits=20, verbose_name='Buy Median'),
        ),
        migrations.AlterField(
            model_name='typestatistics',
            name='buy_weighted_average',
            field=models.DecimalField(decimal_places=2, max_digits=20, verbose_name='Buy Weighted Average'),
        ),
        migrations.AlterField(
            model_name='typestatistics',
            name='sell_median',
            field=models.DecimalField(decimal_places=2, max_digits=20, verbose_name='Sell Median'),
        ),
        migrations.AlterField(
            model_name='typestatistics',
            name='sell_weighted_average',
            field=models.DecimalField(decimal_places=2, max_digits=20, verbose_name='Sell Weighted Average'),
        ),
    ]



from django.apps import AppConfig

from . import __version__


class MarketManagerConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = "marketmanager"
    verbose_name = f'AA Market Manager v{__version__}'



"""
DO NOT EDIT THIS FILE

This settings file contains everything needed for Alliance Auth projects to function.
It gets overwritten by the 'allianceauth update' command.
If you wish to make changes, overload the setting in your project's settings file (local.py).
"""

import os

from celery.schedules import crontab

from django.contrib import messages

INSTALLED_APPS = [
    'allianceauth',  # needs to be on top of this list to support favicons in Django admin (see https://gitlab.com/allianceauth/allianceauth/-/issues/1301)
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.humanize',
    'django_celery_beat',
    'bootstrapform',
    'sortedm2m',
    'esi',
    'allianceauth.authentication',
    'allianceauth.services',
    'allianceauth.eveonline',
    'allianceauth.groupmanagement',
    'allianceauth.notifications',
    'allianceauth.thirdparty.navhelper',
    'allianceauth.analytics',
]

SECRET_KEY = "wow I'm a really bad default secret key"

# Celery configuration
BROKER_URL = 'redis://localhost:6379/0'
CELERYBEAT_SCHEDULER = "django_celery_beat.schedulers.DatabaseScheduler"
CELERYBEAT_SCHEDULE = {
    'esi_cleanup_callbackredirect': {
        'task': 'esi.tasks.cleanup_callbackredirect',
        'schedule': crontab(minute=0, hour='*/4'),
    },
    'esi_cleanup_token': {
        'task': 'esi.tasks.cleanup_token',
        'schedule': crontab(minute=0, hour=0),
    },
    'run_model_update': {
        'task': 'allianceauth.eveonline.tasks.run_model_update',
        'schedule': crontab(minute=0, hour="*/6"),
    },
    'check_all_character_ownership': {
        'task': 'allianceauth.authentication.tasks.check_all_character_ownership',
        'schedule': crontab(minute=0, hour='*/4'),
    },
    'analytics_daily_stats': {
        'task': 'allianceauth.analytics.tasks.analytics_daily_stats',
        'schedule': crontab(minute=0, hour=2),
    }
}

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
PROJECT_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
BASE_DIR = os.path.dirname(PROJECT_DIR)

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'allianceauth.analytics.middleware.AnalyticsMiddleware',
]

ROOT_URLCONF = 'allianceauth.urls'

LOCALE_PATHS = (
    os.path.join(BASE_DIR, 'locale/'),
)

ugettext = lambda s: s
LANGUAGES = (
    ('en', ugettext('English')),
    ('de', ugettext('German')),
    ('es', ugettext('Spanish')),
    ('zh-hans', ugettext('Chinese Simplified')),
    ('ru', ugettext('Russian')),
    ('ko', ugettext('Korean')),
    ('fr', ugettext('French')),
    ('ja', ugettext('Japanese')),
    ('it', ugettext('Italian')),
)

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(PROJECT_DIR, 'templates')],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'allianceauth.context_processors.auth_settings',
            ],
        },
    },
]

WSGI_APPLICATION = 'allianceauth.wsgi.application'

# Password validation
# https://docs.djangoproject.com/en/1.10/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

AUTHENTICATION_BACKENDS = [
    'allianceauth.authentication.backends.StateBackend',
    'django.contrib.auth.backends.ModelBackend'
]

# Internationalization
# https://docs.djangoproject.com/en/1.10/topics/i18n/

LANGUAGE_CODE = 'en-us'

LANGUAGE_COOKIE_AGE = 1209600

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.10/howto/static-files/
STATIC_URL = '/static/'
STATICFILES_DIRS = [
    os.path.join(PROJECT_DIR, 'static'),
]
STATIC_ROOT = os.path.join(BASE_DIR, 'static')

# Bootstrap messaging css workaround
MESSAGE_TAGS = {
    messages.ERROR: 'danger error'
}

CACHES = {
    "default": {
        "BACKEND": "redis_cache.RedisCache",
        "LOCATION": "localhost:6379",
        "OPTIONS": {
            "DB": 1,
        }
    }
}

SESSION_ENGINE = "django.contrib.sessions.backends.cached_db"

DEBUG = True
ALLOWED_HOSTS = ['*']
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': str(os.path.join(BASE_DIR, 'alliance_auth.sqlite3')),
    },
}

SITE_NAME = 'Alliance Auth'

LOGIN_URL = 'auth_login_user'  # view that handles login logic

LOGIN_REDIRECT_URL = 'authentication:dashboard'  # default destination when logging in if no redirect specified
LOGOUT_REDIRECT_URL = 'authentication:dashboard'  # destination after logging out
# Both of these redirects accept values as per the django redirect shortcut
# https://docs.djangoproject.com/en/1.11/topics/http/shortcuts/#redirect
# - url names eg 'authentication:dashboard'
# - relative urls eg '/dashboard'
# - absolute urls eg 'http://example.com/dashboard'

# scopes required on new tokens when logging in. Cannot be blank.
LOGIN_TOKEN_SCOPES = ['publicData']

# number of days email verification links are valid for
ACCOUNT_ACTIVATION_DAYS = 1

ESI_API_URL = 'https://esi.evetech.net/'

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'verbose': {
            'format': "[%(asctime)s] %(levelname)s [%(name)s:%(lineno)s] %(message)s",
            'datefmt': "%d/%b/%Y %H:%M:%S"
        },
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
    },
    'handlers': {
        'log_file': {
            'level': 'INFO',  # edit this line to change logging level to file
            'class': 'logging.handlers.RotatingFileHandler',
            'filename': os.path.join(BASE_DIR, 'allianceauth.log'),
            'formatter': 'verbose',
            'maxBytes': 1024 * 1024 * 5,  # edit this line to change max log file size
            'backupCount': 5,  # edit this line to change number of log backups
        },
        'extension_file': {
            'level': 'INFO',
            'class': 'logging.handlers.RotatingFileHandler',
            'filename': os.path.join(BASE_DIR, 'extensions.log'),
            'formatter': 'verbose',
            'maxBytes': 1024 * 1024 * 5,  # edit this line to change max log file size
            'backupCount': 5,  # edit this line to change number of log backups
        },
        'console': {
            'level': 'DEBUG',  # edit this line to change logging level to console
            'class': 'logging.StreamHandler',
            'formatter': 'verbose',
        },
        'notifications': {  # creates notifications for users with logging_notifications permission
            'level': 'ERROR',  # edit this line to change logging level to notifications
            'class': 'allianceauth.notifications.handlers.NotificationHandler',
            'formatter': 'verbose',
        },
    },
    'loggers': {
        'allianceauth': {
            'handlers': ['log_file', 'console', 'notifications'],
            'level': 'DEBUG',
        },
        'extensions': {
            'handlers': ['extension_file', 'console'],
            'level': 'DEBUG',
        },
        'django': {
            'handlers': ['log_file', 'console'],
            'level': 'ERROR',
        },
        'esi': {
            'handlers': ['log_file', 'console'],
            'level': 'DEBUG',
        },
    }
}

DEFAULT_AUTO_FIELD = "django.db.models.AutoField"

########################################################
# local.py settings

# Every setting in base.py can be overloaded by redefining it here.
# from .base import *

# These are required for Django to function properly. Don't touch.
ROOT_URLCONF = "testauth.urls"
WSGI_APPLICATION = "testauth.wsgi.application"
SECRET_KEY = "t$@h+j#yqhmuy$x7$fkhytd&drajgfsb-6+j9pqn*vj0)gq&-2"

# This is where css/images will be placed for your webserver to read
STATIC_ROOT = "/var/www/testauth/static/"

# Change this to change the name of the auth site displayed
# in page titles and the site header.
SITE_NAME = "testauth"

# Change this to enable/disable debug mode, which displays
# useful error messages but can leak sensitive data.
DEBUG = False

# Add any additional apps to this list.
INSTALLED_APPS += [
    "marketmanager",
    "eveuniverse",]

# Enter credentials to use MySQL/MariaDB. Comment out to use sqlite3
"""
DATABASES['default'] = {
    'ENGINE': 'django.db.backends.mysql',
    'NAME': 'alliance_auth',
    'USER': '',
    'PASSWORD': '',
    'HOST': '127.0.0.1',
    'PORT': '3306',
    'OPTIONS': {'charset': 'utf8mb4'},
}
"""

# Register an application at https://developers.eveonline.com for Authentication
# & API Access and fill out these settings. Be sure to set the callback URL
# to https://example.com/sso/callback substituting your domain for example.com
# Logging in to auth requires the publicData scope (can be overridden through the
# LOGIN_TOKEN_SCOPES setting). Other apps may require more (see their docs).
ESI_SSO_CLIENT_ID = "dummy"
ESI_SSO_CLIENT_SECRET = "dummy"
ESI_SSO_CALLBACK_URL = "http://localhost:8000"

# By default emails are validated before new users can log in.
# It's recommended to use a free service like SparkPost or Elastic Email to send email.
# https://www.sparkpost.com/docs/integrations/django/
# https://elasticemail.com/resources/settings/smtp-api/
# Set the default from email to something like 'noreply@example.com'
# Email validation can be turned off by uncommenting the line below. This
# can break some services.
REGISTRATION_VERIFY_EMAIL = False
EMAIL_HOST = ""
EMAIL_PORT = 587
EMAIL_HOST_USER = ""
EMAIL_HOST_PASSWORD = ""
EMAIL_USE_TLS = True
DEFAULT_FROM_EMAIL = ""

#######################################
# Add any custom settings below here. #
#######################################
